require 'studio_game/game'
require 'studio_game/die'

module StudioGame

    describe Game do

        before do
            $stdout = StringIO.new
            @game = Game.new("Knuckleheads")
            @initial_health = 100
            @rounds = 5
            @player = Player.new("moe", @initial_health)
            @game.add_player(@player)
        end

        it "w00ts player if high number rolled" do
            Die.any_instance.stub(:roll).and_return(5)
            @game.play(@rounds)
            @player.health.should == @initial_health + 15 * @rounds
        end

        it "skips player if medium number rolled" do 
            Die.any_instance.stub(:roll).and_return(3)
            @game.play(@rounds)
            @player.health.should == @initial_health
        end

        it "blams player if low number rolled" do
            Die.any_instance.stub(:roll).and_return(2)
            @game.play(@rounds)
            @player.health.should == @initial_health - 10 * @rounds
        end
        
        it "assigns a treasure for points during a player's turn" do
            game = Game.new("Knuckleheads")
            player = Player.new("moe")
            game.add_player(player)
            game.play(1)
            player.points.should_not be_zero
        end

        it "computes total points as the sum of all player points" do
            game = Game.new("Knuckleheads")
        
            player1 = Player.new("moe")
            player2 = Player.new("larry")

            game.add_player(player1)
            game.add_player(player2)

            player1.found_treasure(Treasure.new(:hammer, 50))
            player1.found_treasure(Treasure.new(:hammer, 50))
            player2.found_treasure(Treasure.new(:crowbar, 400))

            game.total_points.should == 500
        end

        it "yields each found treasure and its total points" do
            @player.found_treasure(Treasure.new(:skillet, 100))
            @player.found_treasure(Treasure.new(:skillet, 100))
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))

            yielded = []
            @player.each_found_treasure do |treasure|
                yielded << treasure
            end

            yielded.should == [
                Treasure.new(:skillet, 200),
                Treasure.new(:hammer, 50),
                Treasure.new(:bottle, 25)
            ]
        end

        context "1 strong and two whimpy players" do
            before do
                @player1 = Player.new("joe", 30)
                @player2 = Player.new("kate", 120)
                @player2.found_treasure(Treasure.new(:hammer, 100))
                @player3 = Player.new("pete", 90)
                @game = Game.new("Knuckleheads")
                @game.add_player(@player1) 
                @game.add_player(@player2) 
                @game.add_player(@player3) 
                $stdout = StringIO.new
            end

            it "shows 1 strong and two whimpy players" do
                @game.print_stats 
                $stdout.string.should == <<EOS
Knuckleheads Statistics:

1 strong players:
Kate (120)

2 whimpy players:
Pete (90)
Joe (30)

Knuckleheads High Scores:
Kate................ 220
Pete................ 90
Joe................. 30

Joe's point totals:
0 grand total points

Kate's point totals:
100 total hammer points
100 grand total points

Pete's point totals:
0 grand total points

100 total points from treasures found
EOS
            end

        end
    end
end
