require 'studio_game/player'
require 'studio_game/treasure_trove'

module StudioGame

    describe Player do

        before do
            $stdout = StringIO.new
            @initial_health = 150
            @player = Player.new("vlad", @initial_health)
        end

        it "has a capitalized name" do
            @player.name.should == "Vlad"
        end 

        it "has an initial health" do
            @player.health.should == @initial_health
        end

        it "has a string representation" do
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.to_s.should == "I'm Vlad with health = 150, points = 100 and score = 250" 
        end

        it "computes a score as the sum of its health and points" do
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.score.should == 250
        end

        it "increases health by 15 when w00ted" do
            @player.w00t
            @player.health.should == @initial_health + 15
        end

        it "decreases health by 10 when blammed" do
            @player.blam
            @player.health.should == @initial_health - 10
        end

        it "computes points as the sum of all treasure points" do
            @player.points.should == 0

            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.points.should == 50

            @player.found_treasure(Treasure.new(:crowbar, 400))
            @player.points.should == 450

            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.points.should == 500
        end

        it "intantiates from csv line" do
            line = "Andrew,123"
            player = Player.from_csv(line)
            player.health.should == 123
            player.name.should == "Andrew"
            player.points.should == 0
        end

        context "with health greater than 100" do
            before do
                @player = Player.new("vlad", 150)
            end

            it "is strong" do
                @player.should be_strong
            end
        end

        context "with health less or equal 100" do
            before do
                @player = Player.new("vlad", 100)
            end

            it "is not strong" do
                @player.should_not be_strong
            end
        end

        context "in a collection of players" do
            before do 
                @player1 = Player.new("moe", 300)
                @player2 = Player.new("larry", 250)
                @player3 = Player.new("curly", 200)

                @player2.found_treasure(Treasure.new(:skillet, 100))
                @player3.found_treasure(Treasure.new(:broomstick, 400))
          
                @players = [@player1, @player2, @player3]
            end

            it "is sorted by decreasing score" do
                @players.sort.should == [@player3, @player2, @player1]
            end
        end
    end

end
